#include <linux/fb.h>
#include <asm/uaccess.h>

#include "linux/ump.h"
#include "linux/ump-common.h"

int mali_venusfb_get_ump_secure_id(struct fb_info *info, unsigned long arg) 
{
	unsigned int __user *psecureid;
	ump_secure_id secure_id;
	ump_dd_physical_block ump_memory_description;
	ump_dd_handle ump_wrapped_buffer;

	psecureid = (unsigned int __user *) arg;
	ump_memory_description.addr = info->fix.smem_start;
	ump_memory_description.size = info->fix.smem_len;
		
	ump_wrapped_buffer = ump_dd_handle_create_from_phys_blocks(&ump_memory_description, 1);

	if (ump_wrapped_buffer == UMP_DD_INVALID_MEMORY_HANDLE)
	{
		printk("mali_venusfb: no ump wrapped_buffer\n");
		return -EINVAL;
	}

	secure_id = ump_dd_secure_id_get(ump_wrapped_buffer);

	//printk("mali_venusfb: ump buffer=%x, secure_id=%x\n", ump_wrapped_buffer, secure_id);

	return put_user((unsigned int)secure_id, psecureid);
}
int mali_venusfb_get_ump_secure_id_offset_idx(struct fb_info *info, unsigned long arg, int addr, int bytes) 
{
	unsigned int __user *psecureid;
	ump_secure_id secure_id;
	ump_dd_physical_block ump_memory_description;
	ump_dd_handle ump_wrapped_buffer;
	
	psecureid = (unsigned int __user *) arg;
	ump_memory_description.addr = addr;
	ump_memory_description.size = bytes;
		
	ump_wrapped_buffer = ump_dd_handle_create_from_phys_blocks(&ump_memory_description, 1);
	
	if (ump_wrapped_buffer == UMP_DD_INVALID_MEMORY_HANDLE)
	{
		printk("mali_venusfb: no ump wrapped_buffer\n");
		return -EINVAL;
	}

	secure_id = ump_dd_secure_id_get(ump_wrapped_buffer);

	//printk("mali_venusfb: ump buffer=%x, secure_id=%x\n", ump_wrapped_buffer, secure_id);

	return put_user((unsigned int)secure_id, psecureid);
}
